@@ -1,6 +1,7 @@
 library(httptest2)
 
-set_osmapi_connection("testing")
 options(osmapir.osmapir_version = "")
-options(osmapir.user_agent = "osmapiR (https://github.com/jmaspons/osmapiR)")
+options(osmapir.user_agent = "osmapiR (https://github.com/ropensci/osmapiR)")
 options(osmapir.R_CMD_check = TRUE)
+
+set_osmapi_connection("testing")
