@@ -11,7 +11,7 @@ class_columns <- list(
 
 ## osm_get_objects ----
 
-test_that("osm_read_object works", {
+test_that("osm_get_objects works", {
   ## Test errors
 
   expect_error(
@@ -51,10 +51,20 @@ test_that("osm_read_object works", {
 
 test_that("osm_read_object works", {
   read <- list()
+  xml_read <- list()
+  json_read <- list()
   with_mock_dir("mock_read_object", {
     read$node <- osm_get_objects(osm_type = "node", osm_id = 35308286)
     read$way <- osm_get_objects(osm_type = "way", osm_id = 13073736L)
     read$rel <- osm_get_objects(osm_type = "relation", osm_id = "40581")
+
+    xml_read$node <- osm_get_objects(osm_type = "node", osm_id = 35308286, format = "xml")
+    xml_read$way <- osm_get_objects(osm_type = "way", osm_id = 13073736L, format = "xml")
+    xml_read$rel <- osm_get_objects(osm_type = "relation", osm_id = "40581", format = "xml")
+
+    json_read$node <- osm_get_objects(osm_type = "node", osm_id = 35308286, format = "json")
+    json_read$way <- osm_get_objects(osm_type = "way", osm_id = 13073736L, format = "json")
+    json_read$rel <- osm_get_objects(osm_type = "relation", osm_id = "40581", format = "json")
   })
 
   lapply(read, expect_s3_class, c("osmapi_objects", "data.frame"))
@@ -64,8 +74,8 @@ test_that("osm_read_object works", {
     })
   })
 
-  expect_identical(names(read$node), column_objects)
-  lapply(read[c("way", "rel")], function(x) expect_identical(names(x), column_objects))
+  expect_named(read$node, column_objects)
+  lapply(read[c("way", "rel")], function(x) expect_named(x, column_objects))
 
   lapply(read, function(x) {
     mapply(function(y, cl) expect_true(inherits(y, cl)), y = x, cl = class_columns[names(x)])
@@ -76,6 +86,19 @@ test_that("osm_read_object works", {
 
   # methods
   lapply(read, function(x) expect_snapshot(print(x)))
+
+
+  lapply(xml_read, expect_s3_class, "xml_document")
+  lapply(json_read, expect_type, "list")
+
+
+  # Compare xml, json & R
+  mapply(function(d, x) {
+    expect_identical(nrow(d), xml2::xml_length(x))
+  }, d = read, x = xml_read)
+  mapply(function(d, x) {
+    expect_identical(nrow(d), length(x$elements))
+  }, d = read, x = json_read)
 })
 
 
@@ -192,10 +215,20 @@ test_that("edit OSM object works", {
 
 test_that("osm_history_object works", {
   history <- list()
+  xml_history <- list()
+  json_history <- list()
   with_mock_dir("mock_history_object", {
     history$node <- osm_history_object(osm_type = "node", osm_id = 35308286)
     history$way <- osm_history_object(osm_type = "way", osm_id = 13073736L)
     history$rel <- osm_history_object(osm_type = "relation", osm_id = "40581")
+
+    xml_history$node <- osm_history_object(osm_type = "node", osm_id = 35308286, format = "xml")
+    xml_history$way <- osm_history_object(osm_type = "way", osm_id = 13073736L, format = "xml")
+    xml_history$rel <- osm_history_object(osm_type = "relation", osm_id = "40581", format = "xml")
+
+    json_history$node <- osm_history_object(osm_type = "node", osm_id = 35308286, format = "json")
+    json_history$way <- osm_history_object(osm_type = "way", osm_id = 13073736L, format = "json")
+    json_history$rel <- osm_history_object(osm_type = "relation", osm_id = "40581", format = "json")
   })
 
   lapply(history, expect_s3_class, c("osmapi_objects", "data.frame"))
@@ -204,13 +237,26 @@ test_that("osm_history_object works", {
       expect_true(is.null(y) | inherits(y, "way_members") | inherits(y, "relation_members"))
     })
   })
-  expect_identical(names(history$node)[seq_len(length(column_objects))], column_objects)
+  expect_named(history$node[, seq_len(length(column_objects))], column_objects)
   lapply(history[c("way", "rel")], function(x) {
-    expect_identical(names(x)[seq_len(length(column_objects))], column_objects)
+    expect_named(x[, seq_len(length(column_objects))], column_objects)
   })
 
   # methods
   lapply(history, function(x) expect_snapshot(print(x)))
+
+
+  lapply(xml_history, expect_s3_class, "xml_document")
+  lapply(json_history, expect_type, "list")
+
+
+  # Compare xml, json & R
+  mapply(function(d, x) {
+    expect_identical(nrow(d), xml2::xml_length(x))
+  }, d = history, x = xml_history)
+  mapply(function(d, x) {
+    expect_identical(nrow(d), length(x$elements))
+  }, d = history, x = json_history)
 })
 
 
@@ -218,10 +264,20 @@ test_that("osm_history_object works", {
 
 test_that("osm_version_object works", {
   version <- list()
+  xml_version <- list()
+  json_version <- list()
   with_mock_dir("mock_version_object", {
     version$node <- osm_get_objects(osm_type = "node", osm_id = 35308286, version = 1)
     version$way <- osm_get_objects(osm_type = "way", osm_id = 13073736L, version = 2)
     version$rel <- osm_get_objects(osm_type = "relation", osm_id = "40581", version = 3)
+
+    xml_version$node <- osm_get_objects(osm_type = "node", osm_id = 35308286, version = 1, format = "xml")
+    xml_version$way <- osm_get_objects(osm_type = "way", osm_id = 13073736L, version = 2, format = "xml")
+    xml_version$rel <- osm_get_objects(osm_type = "relation", osm_id = "40581", version = 3, format = "xml")
+
+    json_version$node <- osm_get_objects(osm_type = "node", osm_id = 35308286, version = 1, format = "json")
+    json_version$way <- osm_get_objects(osm_type = "way", osm_id = 13073736L, version = 2, format = "json")
+    json_version$rel <- osm_get_objects(osm_type = "relation", osm_id = "40581", version = 3, format = "json")
   })
 
   lapply(version, expect_s3_class, c("osmapi_objects", "data.frame"))
@@ -230,13 +286,26 @@ test_that("osm_version_object works", {
       expect_true(is.null(y) | inherits(y, "way_members") | inherits(y, "relation_members"))
     })
   })
-  expect_identical(names(version$node)[seq_len(length(column_objects))], column_objects)
+  expect_named(version$node[, seq_len(length(column_objects))], column_objects)
   lapply(version[c("way", "rel")], function(x) {
-    expect_identical(names(x)[seq_len(length(column_objects))], column_objects)
+    expect_named(x[, seq_len(length(column_objects))], column_objects)
   })
 
   # methods
   lapply(version, function(x) expect_snapshot(print(x)))
+
+
+  lapply(xml_version, expect_s3_class, "xml_document")
+  lapply(json_version, expect_type, "list")
+
+
+  # Compare xml, json & R
+  mapply(function(d, x) {
+    expect_identical(nrow(d), xml2::xml_length(x))
+  }, d = version, x = xml_version)
+  mapply(function(d, x) {
+    expect_identical(nrow(d), length(x$elements))
+  }, d = version, x = json_version)
 })
 
 
@@ -266,9 +335,9 @@ test_that("osm_fetch_objects works", {
       expect_true(is.null(y) | inherits(y, "way_members") | inherits(y, "relation_members"))
     })
   })
-  expect_identical(names(fetch$node)[seq_len(length(column_objects))], column_objects)
+  expect_named(fetch$node[, seq_len(length(column_objects))], column_objects)
   lapply(fetch[c("way", "rel")], function(x) {
-    expect_identical(names(x)[seq_len(length(column_objects))], column_objects)
+    expect_named(x[, seq_len(length(column_objects))], column_objects)
   })
 
   lapply(fetch_xml, expect_s3_class, "xml_document")
@@ -402,6 +471,11 @@ test_that("osm_fetch_objects works", {
     expect_identical(id, osm_ids)
     expect_false(any(duplicated(id)))
   })
+
+  mapply(function(d, x) {
+    expect_identical(nrow(d), xml2::xml_length(x))
+  }, d = fetch_many[1:2], x = fetch_many_xml)
+
   expect_message(set_osmapi_connection("testing"), "Logged out from") # TODO
 
   # methods
@@ -413,10 +487,20 @@ test_that("osm_fetch_objects works", {
 
 test_that("osm_relations_object works", {
   rels <- list()
+  xml_rels <- list()
+  json_rels <- list()
   with_mock_dir("mock_relations_object", {
     rels$node <- osm_relations_object(osm_type = "node", osm_id = 1470837704)
     rels$way <- osm_relations_object(osm_type = "way", osm_id = 372011578)
     rels$rel <- osm_relations_object(osm_type = "relation", osm_id = 342792)
+
+    xml_rels$node <- osm_relations_object(osm_type = "node", osm_id = 1470837704, format = "xml")
+    xml_rels$way <- osm_relations_object(osm_type = "way", osm_id = 372011578, format = "xml")
+    xml_rels$rel <- osm_relations_object(osm_type = "relation", osm_id = 342792, format = "xml")
+
+    json_rels$node <- osm_relations_object(osm_type = "node", osm_id = 1470837704, format = "json")
+    json_rels$way <- osm_relations_object(osm_type = "way", osm_id = 372011578, format = "json")
+    json_rels$rel <- osm_relations_object(osm_type = "relation", osm_id = 342792, format = "json")
   })
 
   lapply(rels, expect_s3_class, c("osmapi_objects", "data.frame"))
@@ -425,10 +509,23 @@ test_that("osm_relations_object works", {
       expect_true(is.null(y) | inherits(y, "way_members") | inherits(y, "relation_members"))
     })
   })
-  lapply(rels, function(x) expect_identical(names(x)[seq_len(length(column_objects))], column_objects))
+  lapply(rels, function(x) expect_named(x[, seq_len(length(column_objects))], column_objects))
 
   # methods
   lapply(rels, function(x) expect_snapshot(print(x)))
+
+
+  lapply(xml_rels, expect_s3_class, "xml_document")
+  lapply(json_rels, expect_type, "list")
+
+
+  # Compare xml, json & R
+  mapply(function(d, x) {
+    expect_identical(nrow(d), xml2::xml_length(x))
+  }, d = rels, x = xml_rels)
+  mapply(function(d, x) {
+    expect_identical(nrow(d), length(x$elements))
+  }, d = rels, x = json_rels)
 })
 
 
@@ -437,16 +534,27 @@ test_that("osm_relations_object works", {
 test_that("osm_ways_node works", {
   with_mock_dir("mock_ways_node", {
     ways_node <- osm_ways_node(node_id = 35308286)
+    xml_ways_node <- osm_ways_node(node_id = 35308286, format = "xml")
+    json_ways_node <- osm_ways_node(node_id = 35308286, format = "json")
   })
 
   expect_s3_class(ways_node, c("osmapi_objects", "data.frame"))
   lapply(ways_node$members, function(x) {
     expect_true(is.null(x) | inherits(x, "way_members") | inherits(x, "relation_members"))
   })
-  expect_identical(names(ways_node)[seq_len(length(column_objects))], column_objects)
+  expect_named(ways_node[, seq_len(length(column_objects))], column_objects)
 
   # methods
   expect_snapshot(print(ways_node))
+
+
+  expect_s3_class(xml_ways_node, "xml_document")
+  expect_type(json_ways_node, "list")
+
+
+  # Compare xml, json & R
+  expect_identical(nrow(ways_node), xml2::xml_length(xml_ways_node))
+  expect_identical(nrow(ways_node), length(json_ways_node$elements))
 })
 
 
@@ -457,11 +565,22 @@ test_that("osm_full_object works", {
   with_mock_dir("mock_full_object", {
     full$way <- osm_get_objects(osm_type = "way", osm_id = 13073736, full_objects = TRUE)
     full$rel <- osm_get_objects(osm_type = "relation", osm_id = "6002785", full_objects = TRUE)
+
+    full$df <- osm_get_objects(
+      osm_type = c("relation", "way", "way", "node"),
+      osm_id = c(6002785, 13073736, 235744929, 35308286),
+      full_objects = TRUE, format = "R"
+    )
     full_xml <- osm_get_objects(
       osm_type = c("relation", "way", "way", "node"),
       osm_id = c(6002785, 13073736, 235744929, 35308286),
       full_objects = TRUE, format = "xml"
     )
+    full_json <- osm_get_objects(
+      osm_type = c("relation", "way", "way", "node"),
+      osm_id = c(6002785, 13073736, 235744929, 35308286),
+      full_objects = TRUE, format = "json"
+    )
   })
 
   lapply(full, expect_s3_class, c("osmapi_objects", "data.frame"))
@@ -470,29 +589,30 @@ test_that("osm_full_object works", {
       expect_true(is.null(y) | inherits(y, "way_members") | inherits(y, "relation_members"))
     })
   })
-  lapply(full, function(x) expect_identical(names(x)[seq_len(length(column_objects))], column_objects))
+  lapply(full, function(x) expect_named(x[, seq_len(length(column_objects))], column_objects))
 
   # methods
   lapply(full, function(x) expect_snapshot(print(x)))
 
 
-  ## xml
   expect_s3_class(full_xml, "xml_document")
 
-
-  ## json
-  with_mock_dir("mock_full_object_json", {
-    full_json <- osm_get_objects(
-      osm_type = c("relation", "way", "way", "node"),
-      osm_id = c(6002785, 13073736, 235744929, 35308286),
-      full_objects = TRUE, format = "json"
-    )
-  })
   expect_type(full_json, "list")
   expect_named(full_json, c("version", "generator", "copyright", "attribution", "license", "elements"))
   lapply(full_json$elements, function(x) {
     expect_contains(names(x), c("type", "id", "timestamp", "version", "changeset", "user", "uid"))
   })
+
+
+  # Compare xml, json & R
+  expect_identical(nrow(full$df), xml2::xml_length(full_xml))
+  expect_identical(nrow(full$df), length(full_json$elements))
+
+  id_df <- full$df$id
+  id_xml <- sapply(xml2::xml_children(full_xml), xml2::xml_attr, attr = "id")
+  id_json <- as.character(sapply(full_json$elements, function(x) x$id))
+  expect_setequal(id_df, id_xml)
+  expect_setequal(id_df, id_json)
 })
 
 
